/*
 * FindBugs - Find bugs in Java programs
 *
 * (C) Copyright 2005 Panagiotis Louridas <louridas@aueb.gr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: XPathV.java,v 1.1 2005/07/31 22:43:52 panos Exp $
 */

package edu.umd.cs.findbugs.apiv.javax.xml.xpath;
import edu.umd.cs.findbugs.apiv.VerifyResult;


/**
 * Generic API invocation verifier for javax.xml.xpath.XPath
 * Return SYNTAX_ERROR for invalid xpath expressions, and
 * ARGUMENT_ERROR in a number of other cases.
 *
 * @see edu.umd.cs.findbugs.detect.ApiVerify
 * @author Panagiotis Louridas 
 * @version $Revision: 1.1 $
 */

import javax.xml.xpath.XPath;
import javax.xml.namespace.QName;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPathExpressionException;
import org.xml.sax.InputSource;
import org.w3c.dom.Document;
import java.io.StringReader;

public class XPathV {

    private static final String DUMMY_XML = "<xml/>";
    
    public VerifyResult.Value evaluate(boolean isConstant[], XPath xpath,
            String expression, InputSource inputSource) {
	if (!isConstant[1])
            return VerifyResult.Value.UNVERIFIED;
        try {
            XPath tester = xpath == null
                ? XPathFactory.newInstance().newXPath()
                : xpath;
            tester.evaluate(expression,
                new InputSource(new StringReader(DUMMY_XML)));
            return VerifyResult.Value.CORRECT;
        } catch (XPathExpressionException xpex) {
            return VerifyResult.Value.SYNTAX_ERROR;
        } catch (NullPointerException npex) {
	    return VerifyResult.Value.ARGUMENT_ERROR;
        }
    }

    public VerifyResult.Value evaluate(boolean isConstant[], XPath xpath,
            String expression, InputSource inputSource, QName returnType) {
        return evaluate(isConstant, xpath, expression, inputSource);
    }

    public VerifyResult.Value evaluate(boolean isConstant[], XPath xpath,
            String expression, Object item) {
	if (!isConstant[1])
            return VerifyResult.Value.UNVERIFIED;
        try {
            XPath tester = xpath == null
                ? XPathFactory.newInstance().newXPath()
                : xpath;
            tester.evaluate(expression,
                new InputSource(new StringReader(DUMMY_XML)));
            return VerifyResult.Value.CORRECT;
        } catch (XPathExpressionException xpex) {
            return VerifyResult.Value.SYNTAX_ERROR;
        } catch (NullPointerException npex) {
	    return VerifyResult.Value.ARGUMENT_ERROR;
        }
    }

    public VerifyResult.Value evaluate(boolean isConstant[], XPath xpath, 
            String expression, Object item, QName returnType) {
        return evaluate(isConstant, xpath, expression, item);
    }
}
