/*
 * FindBugs - Find bugs in Java programs
 *
 * (C) Copyright 2005 Panagiotis Louridas <louridas@aueb.gr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: NodeV.java,v 1.1 2005/08/05 16:15:17 panos Exp $
 */

package edu.umd.cs.findbugs.apiv.org.dom4j;
import edu.umd.cs.findbugs.apiv.VerifyResult;

/**
 * Generic API invocation verifier for org.dom4j.Node
 * Return SYNTAX_ERROR for invalid xpath expressions, and
 * ARGUMENT_ERROR in a number of other cases.
 *
 * @see edu.umd.cs.findbugs.detect.ApiVerify
 * @author Panagiotis Louridas 
 * @version $Revision: 1.1 $
 */

import org.dom4j.Node;

import org.xml.sax.InputSource;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPathExpressionException;

import java.io.StringReader;

public class NodeV {

    private static final String DUMMY_XML = "<xml/>";

    public VerifyResult.Value matches(boolean isConstant[], Node node,
        String expr) {
        if (!isConstant[1])
            return VerifyResult.Value.UNVERIFIED;
        try {
            javax.xml.xpath.XPath tester =
                XPathFactory.newInstance().newXPath();
            tester.evaluate(expr,
                new InputSource(new StringReader(DUMMY_XML)));
            return VerifyResult.Value.CORRECT;
        } catch (XPathExpressionException xpex) {
            return VerifyResult.Value.SYNTAX_ERROR;
        } catch (NullPointerException npex) {
	    return VerifyResult.Value.ARGUMENT_ERROR;
        }
    }
    
    public VerifyResult.Value numberValueOf(boolean isConstant[], Node node,
        String expr) {
        return matches(isConstant, node, expr);
    }

    public VerifyResult.Value selectNodes(boolean isConstant[], Node node,
        String expr) {
        return matches(isConstant, node, expr);
    }
                       
    public VerifyResult.Value selectNodes(boolean isConstant[], Node node,
        String expr, String comparisonExpression) {
        VerifyResult.Value firstArg = matches(isConstant, node, expr);
        if (firstArg == VerifyResult.Value.CORRECT
            || firstArg == VerifyResult.Value.UNVERIFIED) {
            boolean isConstantC[] = new boolean[2];
            isConstantC[0] = isConstant[0];
            isConstantC[1] = isConstant[2];
            return matches(isConstantC, node, comparisonExpression);
        } else
            return firstArg;
    }

    public VerifyResult.Value selectNodes(boolean isConstant[], Node node,
        String expr, String comparisonExpression, boolean removeDublicates) {
        return selectNodes(isConstant, node, expr, comparisonExpression);
    }

    public VerifyResult.Value selectObject(boolean isConstant[], Node node,
        String expr) {
        return matches(isConstant, node, expr);
    }

    public VerifyResult.Value selectSingleNode(boolean isConstant[], Node node,
        String expr) {
        return matches(isConstant, node, expr);
    }
    
    public VerifyResult.Value valueOf(boolean isConstant[], Node node,
        String expr) {
        return matches(isConstant, node, expr);
    }
}
